searchData={"items":[{"type":"module","title":"LazyHTML","doc":"Efficient parsing and querying of HTML documents.\n\nLazyHTML is designed around lazy HTML documents. Documents are parsed\nand kept natively in memory for as long as possible. Query selectors\nare executed in native code for performance and adheres to browser standards.\nUnder the hood, LazyHTML uses [Lexbor](https://github.com/lexbor/lexbor),\na fast, dependency-free and comprehensive HTML engine, written entirely in C.\n\nLazyHTML works with a flat list of nodes and all operations are batched\nby default, as shown below:\n\n```elixir\nlazy_html =\n  LazyHTML.from_fragment(\"\"\"\n   \n     Elixir \n     Erlang \n   \\\n  \"\"\")\n#=> #LazyHTML<\n#=>   1 node\n#=>\n#=>   #1\n#=>    \n#=>      Elixir \n#=>      Erlang \n#=>    \n#=> >\n\nhyperlinks = LazyHTML.query(lazy_html, \"a\")\n#=> #LazyHTML<\n#=>   2 nodes (from selector)\n#=>\n#=>   #1\n#=>    Elixir \n#=>\n#=>   #2\n#=>    Erlang \n#=> >\n\nLazyHTML.attribute(hyperlinks, \"href\")\n#=> [\"https://elixir-lang.org\", \"https://www.erlang.org\"]\n```\n\nLazyHTML also provides several high-level conveniences:\n\n- an `Inspect` implementation to pretty-print nodes\n- an `Access` implementation to run CSS selectors\n- an `Enumerable` implementation to traverse them\n\nFor example:\n\n```elixir\nlazy_html = LazyHTML.from_fragment(~S|  Hello ,  world ! |)\n#=> #LazyHTML<\n#=>   1 node\n#=>\n#=>   #1\n#=>     Hello ,  world ! \n#=> >\n\nlazy_html[\"strong, em\"]\n#=> #LazyHTML<\n#=>   2 nodes (from selector)\n#=>\n#=>   #1\n#=>    Hello \n#=>\n#=>   #2\n#=>    world \n#=> >\n\nLazyHTML.text(lazy_html)\n#=> \"Hello, world!\"\n\nEnum.map(lazy_html[\"strong, em\"], &LazyHTML.text/1)\n#=> [\"Hello\", \"world\"]\n```\n\nIf needed, the lazy nodes can be converted into an Elixir tree data\nstructure, and vice-versa.\n\n```elixir\nlazy_html = LazyHTML.from_fragment(\"  Hello ,  world ! \")\n#=> #LazyHTML<\n#=>   1 node\n#=>\n#=>   #1\n#=>     Hello ,  world ! \n#=> >\n\ntree = LazyHTML.to_tree(lazy_html)\n#=> [{\"p\", [], [{\"strong\", [], [\"Hello\"]}, \", \", {\"em\", [], [\"world\"]}, \"!\"]}]\n\nLazyHTML.from_tree(tree)\n#=> #LazyHTML<\n#=>   1 node\n\n#=>   #1\n#=>     Hello ,  world ! \n#=> >\n```","ref":"LazyHTML.html"},{"type":"function","title":"LazyHTML.attribute/2","doc":"Returns all values of the given attribute on the `lazy_html` root\nnodes.","ref":"LazyHTML.html#attribute/2"},{"type":"function","title":"Examples - LazyHTML.attribute/2","doc":"iex> lazy_html =\n    ...>   LazyHTML.from_fragment(\"\"\"\n    ...>    \n    ...>      Hello \n    ...>      world \n    ...>      ! \n    ...>    \n    ...>   \"\"\")\n    iex> spans = LazyHTML.query(lazy_html, \"span\")\n    iex> LazyHTML.attribute(spans, \"data-id\")\n    [\"1\", \"2\"]\n    iex> LazyHTML.attribute(spans, \"data-other\")\n    []\n\nNote that attributes without value, implicitly have an empty value:\n\n    iex> lazy_html = LazyHTML.from_fragment(~S|  Click me  |)\n    iex> button = LazyHTML.query(lazy_html, \"button\")\n    iex> LazyHTML.attribute(button, \"disabled\")\n    [\"\"]","ref":"LazyHTML.html#attribute/2-examples"},{"type":"function","title":"LazyHTML.attributes/1","doc":"Returns attribute lists for every root element in `lazy_html`.\n\nNote that if there are text or comment root nodes, they are ignored,\nand they have no corresponding list in the result.","ref":"LazyHTML.html#attributes/1"},{"type":"function","title":"Examples - LazyHTML.attributes/1","doc":"iex> lazy_html =\n    ...>   LazyHTML.from_fragment(\"\"\"\n    ...>    \n    ...>      Hello \n    ...>      world \n    ...>    \n    ...>   \"\"\")\n    iex> spans = LazyHTML.query(lazy_html, \"span\")\n    iex> LazyHTML.attributes(spans)\n    [\n      [{\"class\", \"text\"}, {\"data-id\", \"1\"}],\n      []\n    ]\n\n    iex> lazy_html =\n    ...>   LazyHTML.from_fragment(\"\"\"\n    ...>   <!-- Comment-->\n    ...>    Hello \n    ...>   world\n    ...>   \"\"\")\n    iex> LazyHTML.attributes(lazy_html)\n    [\n      [{\"class\", \"text\"}]\n    ]","ref":"LazyHTML.html#attributes/1-examples"},{"type":"function","title":"LazyHTML.child_nodes/1","doc":"Returns the child_nodes nodes of the root nodes in `lazy_html`.","ref":"LazyHTML.html#child_nodes/1"},{"type":"function","title":"Examples - LazyHTML.child_nodes/1","doc":"iex> lazy_html = LazyHTML.from_fragment(~S|  Hello   world  |)\n    iex> LazyHTML.child_nodes(lazy_html)\n    #LazyHTML<\n      3 nodes (from selector)\n      #1\n       Hello \n      #2\n      [whitespace]\n      #3\n       world \n    >\n    iex> LazyHTML.child_nodes(LazyHTML.child_nodes(lazy_html))\n    #LazyHTML<\n      2 nodes (from selector)\n      #1\n      Hello\n      #2\n      world\n    >","ref":"LazyHTML.html#child_nodes/1-examples"},{"type":"function","title":"LazyHTML.filter/2","doc":"Filters `lazy_html` root nodes, keeping only elements that match\nthe given CSS selector.","ref":"LazyHTML.html#filter/2"},{"type":"function","title":"Examples - LazyHTML.filter/2","doc":"iex> lazy_html = LazyHTML.from_fragment(\"\"\"\n    ...>  Hello \n    ...>  \n    ...>    nested \n    ...>  \n    ...>  world \n    ...> \"\"\")\n    iex> LazyHTML.filter(lazy_html, \"span\")\n    #LazyHTML<\n      2 nodes (from selector)\n      #1\n       Hello \n      #2\n       world \n    >","ref":"LazyHTML.html#filter/2-examples"},{"type":"function","title":"LazyHTML.from_document/1","doc":"Parses an HTML document.\n\nThis function expects a complete document, therefore if either of\n` `, ` ` or ` ` tags is missing, it will be added,\nwhich matches the usual browser behaviour. To parse a part of an\nHTML document, use `from_fragment/1` instead.","ref":"LazyHTML.html#from_document/1"},{"type":"function","title":"Examples - LazyHTML.from_document/1","doc":"iex> LazyHTML.from_document(~S|    Hello world!  |)\n    #LazyHTML<\n      1 node\n      #1\n          Hello world!  \n    >\n\n    iex> LazyHTML.from_document(~S| Hello world! |)\n    #LazyHTML<\n      1 node\n      #1\n           Hello world!   \n    >","ref":"LazyHTML.html#from_document/1-examples"},{"type":"function","title":"LazyHTML.from_fragment/1","doc":"Parses a segment of an HTML document.\n\nAs opposed to `from_document/1`, this function does not expect a full\ndocument and does not add any extra tags.","ref":"LazyHTML.html#from_fragment/1"},{"type":"function","title":"Examples - LazyHTML.from_fragment/1","doc":"iex> LazyHTML.from_fragment(~S| Click me |)\n    #LazyHTML<\n      1 node\n      #1\n       Click me \n    >\n\n    iex> LazyHTML.from_fragment(~S| Hello   world |)\n    #LazyHTML<\n      3 nodes\n      #1\n       Hello \n      #2\n      [whitespace]\n      #3\n       world \n    >","ref":"LazyHTML.html#from_fragment/1-examples"},{"type":"function","title":"LazyHTML.from_tree/1","doc":"Builds a lazy HTML document from an Elixir tree data structure.","ref":"LazyHTML.html#from_tree/1"},{"type":"function","title":"Examples - LazyHTML.from_tree/1","doc":"iex> tree = [\n    ...>   {\"html\", [], [{\"head\", [], [{\"title\", [], [\"Page\"]}]}, {\"body\", [], [\"Hello world\"]}]}\n    ...> ]\n    iex> LazyHTML.from_tree(tree)\n    #LazyHTML<\n      1 node\n      #1\n         Page   Hello world  \n    >\n\n    iex> tree = [\n    ...>   {\"div\", [], []},\n    ...>   {:comment, \" Link \"},\n    ...>   {\"a\", [{\"href\", \"https://elixir-lang.org\"}], [\"Elixir\"]}\n    ...> ]\n    iex> LazyHTML.from_tree(tree)\n    #LazyHTML<\n      3 nodes\n      #1\n        \n      #2\n      <!-- Link -->\n      #3\n       Elixir \n    >","ref":"LazyHTML.html#from_tree/1-examples"},{"type":"function","title":"LazyHTML.query/2","doc":"Finds elements in `lazy_html` matching the given CSS selector.\n\nSince `lazy_html` may have multiple root nodes, the root nodes are\nincluded in the search and they will appear in the result if they\nmatch the given selector.","ref":"LazyHTML.html#query/2"},{"type":"function","title":"Examples - LazyHTML.query/2","doc":"iex> lazy_html =\n    ...>   LazyHTML.from_fragment(\"\"\"\n    ...>    \n    ...>      Hello \n    ...>      world \n    ...>    \n    ...>   \"\"\")\n    iex> LazyHTML.query(lazy_html, \"span\")\n    #LazyHTML<\n      2 nodes (from selector)\n      #1\n       Hello \n      #2\n       world \n    >\n    iex> LazyHTML.query(lazy_html, \".layout\")\n    #LazyHTML<\n      1 node (from selector)\n      #1\n       \n         Hello \n         world \n       \n    >\n\nNote that for each root node, the selector respects its actual\nlocation in the document. Consequently, if you run one `query/2`\nthe returned nodes are not necessarily siblings, which may impact\na subsequent query:\n\n    iex> lazy_html =\n    ...>   LazyHTML.from_fragment(\"\"\"\n    ...>    \n    ...>      Hello \n    ...>    \n    ...>    \n    ...>      World \n    ...>    \n    ...>   \"\"\")\n    iex> spans = LazyHTML.query(lazy_html, \"span\")\n    #LazyHTML<\n      2 nodes (from selector)\n      #1\n       Hello \n      #2\n       World \n    >\n    iex> LazyHTML.query(spans, \":first-child\")\n    #LazyHTML<\n      2 nodes (from selector)\n      #1\n       Hello \n      #2\n       World \n    >\n\nIn the example above, each of the spans is first child of its\nrespective parent, so the second query matches both.","ref":"LazyHTML.html#query/2-examples"},{"type":"function","title":"LazyHTML.query_by_id/2","doc":"Finds elements in `lazy_html` matching the given id.\n\nThis function is similar to `query/2`, but it accepts unescaped id\nstring.\n\nNote that while technically there should be only a single element\nwith the given id, if there are multiple elements, all of them are\nincluded in the result.","ref":"LazyHTML.html#query_by_id/2"},{"type":"function","title":"Examples - LazyHTML.query_by_id/2","doc":"iex> lazy_html =\n    ...>   LazyHTML.from_fragment(\"\"\"\n    ...>    \n    ...>      Hello \n    ...>      world \n    ...>    \n    ...>   \"\"\")\n    iex> LazyHTML.query_by_id(lazy_html, \"hello\")\n    #LazyHTML<\n      1 node (from selector)\n      #1\n       Hello \n    >","ref":"LazyHTML.html#query_by_id/2-examples"},{"type":"type","title":"LazyHTML.t/0","doc":"","ref":"LazyHTML.html#t:t/0"},{"type":"function","title":"LazyHTML.tag/1","doc":"Returns tag name for every root element in `lazy_html`.\n\nNote that if there are text or comment root nodes, they are ignored,\nand they have no corresponding list in the result.","ref":"LazyHTML.html#tag/1"},{"type":"function","title":"Examples - LazyHTML.tag/1","doc":"iex> lazy_html = LazyHTML.from_fragment(~S|  Hello   world  |)\n    iex> LazyHTML.tag(lazy_html)\n    [\"div\"]\n\n    iex> lazy_html = LazyHTML.from_fragment(~S| Hello   world |)\n    iex> LazyHTML.tag(lazy_html)\n    [\"span\", \"span\"]","ref":"LazyHTML.html#tag/1-examples"},{"type":"function","title":"LazyHTML.text/1","doc":"Returns the text content of all nodes in `lazy_html`.","ref":"LazyHTML.html#text/1"},{"type":"function","title":"Examples - LazyHTML.text/1","doc":"iex> lazy_html = LazyHTML.from_fragment(~S|  Hello   world  |)\n    iex> LazyHTML.text(lazy_html)\n    \"Hello world\"\n\nIf you want to get the text for each root node separately, you can\nuse `Enum.map/2`:\n\n    iex> lazy_html = LazyHTML.from_fragment(~S|  Hello   world  |)\n    iex> spans = LazyHTML.query(lazy_html, \"span\")\n    #LazyHTML<\n      2 nodes (from selector)\n      #1\n       Hello \n      #2\n       world \n    >\n    iex> Enum.map(spans, &LazyHTML.text/1)\n    [\"Hello\", \"world\"]","ref":"LazyHTML.html#text/1-examples"},{"type":"function","title":"LazyHTML.to_html/2","doc":"Serializes `lazy_html` as an HTML string.","ref":"LazyHTML.html#to_html/2"},{"type":"function","title":"Options - LazyHTML.to_html/2","doc":"* `:skip_whitespace_nodes` - when `true`, ignores text nodes that\n    consist entirely of whitespace, usually whitespace between tags.\n    Defaults to `false`.","ref":"LazyHTML.html#to_html/2-options"},{"type":"function","title":"Examples - LazyHTML.to_html/2","doc":"iex> lazy_html = LazyHTML.from_document(~S|    Hello world!  |)\n    iex> LazyHTML.to_html(lazy_html)\n    \"    Hello world!  \"\n\n    iex> lazy_html = LazyHTML.from_fragment(~S| Hello   world |)\n    iex> LazyHTML.to_html(lazy_html)\n    \" Hello   world \"\n\n    iex> lazy_html =\n    ...>   LazyHTML.from_fragment(\"\"\"\n    ...>    \n    ...>       Hello  \n    ...>       world  \n    ...>    \n    ...>   \"\"\")\n    iex> LazyHTML.to_html(lazy_html, skip_whitespace_nodes: true)\n    \"   Hello    world   \"","ref":"LazyHTML.html#to_html/2-examples"},{"type":"function","title":"LazyHTML.to_tree/2","doc":"Builds an Elixir tree data structure representing the `lazy_html`\ndocument.","ref":"LazyHTML.html#to_tree/2"},{"type":"function","title":"Options - LazyHTML.to_tree/2","doc":"* `:sort_attributes` - when `true`, attributes lists are sorted\n    alphabetically by name. Defaults to `false`.","ref":"LazyHTML.html#to_tree/2-options"},{"type":"function","title":"Examples - LazyHTML.to_tree/2","doc":"iex> lazy_html = LazyHTML.from_document(~S|   Page   Hello world  |)\n    iex> LazyHTML.to_tree(lazy_html)\n    [{\"html\", [], [{\"head\", [], [{\"title\", [], [\"Page\"]}]}, {\"body\", [], [\"Hello world\"]}]}]\n\n    iex> lazy_html = LazyHTML.from_fragment(~S| <!-- Link --> Elixir  |)\n    iex> LazyHTML.to_tree(lazy_html)\n    [\n      {\"div\", [], [{:comment, \" Link \"}, {\"a\", [{\"href\", \"https://elixir-lang.org\"}], [\"Elixir\"]}]}\n    ]\n\nYou can get a normalized tree by passing `sort_attributes: true`:\n\n    iex> lazy_html = LazyHTML.from_fragment(~S|  |)\n    iex> LazyHTML.to_tree(lazy_html, sort_attributes: true)\n    [{\"div\", [{\"class\", \"layout\"}, {\"id\", \"root\"}], []}]","ref":"LazyHTML.html#to_tree/2-examples"},{"type":"module","title":"LazyHTML.Tree","doc":"This module deals with HTML documents represented as an Elixir tree\ndata structure.","ref":"LazyHTML.Tree.html"},{"type":"type","title":"LazyHTML.Tree.html_attribute/0","doc":"","ref":"LazyHTML.Tree.html#t:html_attribute/0"},{"type":"type","title":"LazyHTML.Tree.html_comment/0","doc":"","ref":"LazyHTML.Tree.html#t:html_comment/0"},{"type":"type","title":"LazyHTML.Tree.html_node/0","doc":"","ref":"LazyHTML.Tree.html#t:html_node/0"},{"type":"type","title":"LazyHTML.Tree.html_tag/0","doc":"","ref":"LazyHTML.Tree.html#t:html_tag/0"},{"type":"type","title":"LazyHTML.Tree.html_text/0","doc":"","ref":"LazyHTML.Tree.html#t:html_text/0"},{"type":"function","title":"LazyHTML.Tree.postwalk/2","doc":"Same a `postwalk/3`, but with no accumulator.","ref":"LazyHTML.Tree.html#postwalk/2"},{"type":"function","title":"LazyHTML.Tree.postwalk/3","doc":"Performs a depth-first, post-order traversal of the given tree.\n\nThe mapper `fun` can return a list of nodes to replace the given\nnode. In order to remove a node, return an empty list.","ref":"LazyHTML.Tree.html#postwalk/3"},{"type":"type","title":"LazyHTML.Tree.t/0","doc":"","ref":"LazyHTML.Tree.html#t:t/0"},{"type":"function","title":"LazyHTML.Tree.to_html/2","doc":"Serializes Elixir tree data structure as an HTML string.","ref":"LazyHTML.Tree.html#to_html/2"},{"type":"function","title":"Options - LazyHTML.Tree.to_html/2","doc":"* `:skip_whitespace_nodes` - when `true`, ignores text nodes that\n    consist entirely of whitespace, usually whitespace between tags.\n    Defaults to `false`.","ref":"LazyHTML.Tree.html#to_html/2-options"},{"type":"function","title":"Examples - LazyHTML.Tree.to_html/2","doc":"iex> tree = [\n    ...>   {\"html\", [], [{\"head\", [], [{\"title\", [], [\"Page\"]}]}, {\"body\", [], [\"Hello world\"]}]}\n    ...> ]\n    iex> LazyHTML.Tree.to_html(tree)\n    \"   Page   Hello world  \"\n\n    iex> tree = [\n    ...>   {\"div\", [], []},\n    ...>   {:comment, \" Link \"},\n    ...>   {\"a\", [{\"href\", \"https://elixir-lang.org\"}], [\"Elixir\"]}\n    ...> ]\n    iex> LazyHTML.Tree.to_html(tree)\n    ~S|  <!-- Link --> Elixir |\n\n    iex> tree = [\n    ...>   {\"p\", [],\n    ...>    [\n    ...>      \"\\n  \",\n    ...>      {\"span\", [], [\" Hello \"]},\n    ...>      \"\\n  \",\n    ...>      {\"span\", [], [\" world \"]},\n    ...>      \"\\n\"\n    ...>    ]},\n    ...>   \"\\n\"\n    ...> ]\n    iex> LazyHTML.Tree.to_html(tree, skip_whitespace_nodes: true)\n    \"   Hello    world   \"","ref":"LazyHTML.Tree.html#to_html/2-examples"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}